@@ -34,6 +34,7 @@ from azure.mgmt.compute.models import (
     OperatingSystemTypes
 )
 from azure.mgmt.resource.resources.models import Deployment, DeploymentMode, DeploymentProperties
+from msrestazure.tools import parse_resource_id
 from six.moves import range
 
 import clusterondemand.brightsetup
@@ -148,6 +149,30 @@ config_ns.add_parameter(
     validation=requires_other_parameters_to_be_set(["vnet_resource_group", "vnet_network_name"]),
 )
 
+
+def internal_subnet_id_validation(subnet_id_param, config):
+    assert subnet_id_param.key == "internal_subnet_id"
+
+    value = config[subnet_id_param.key]
+    if value:
+        try:
+            parsed = parse_resource_id(value)
+            config["vnet_resource_group"] = parsed["resource_group"]
+            config["vnet_network_name"] = parsed["name"]
+            config["vnet_subnet_name"] = parsed["child_name_1"]
+        except Exception as e:
+            raise CODException(f"'{value}' is not a valid Azure subnet ID") from e
+
+
+config_ns.add_parameter(
+    "internal_subnet_id",
+    advanced=True,
+    help="Azure ID for the existing subnet to be used. If set, a new VNet *will not* be created for this cluster. "
+         "Instead, the head node will be attached to this existing subnet. This parameter combines "
+         "--vnet-resource-group, --vnet-network-name and --vnet-subnet-name. See also --head-node-ip",
+    validation=internal_subnet_id_validation,
+)
+
 config_ns.add_parameter(
     "resource_group",
     advanced=True,
@@ -163,6 +188,15 @@ config_ns.add_switch_parameter(
          "Use --resource-group to set a name.",
 )
 
+config_ns.add_switch_parameter(
+    "create_public_ip",
+    advanced=True,
+    default=True,
+    help="Create a public IP for the head node. Use --no-create-public-ip to skip creating one. In that case, to "
+    "have access to the cluster, it will need to be created within an existing network that is accessible "
+    "by the user. See --vnet-resource-group, --vnet-network-name and --vnet-subnet-name",
+)
+
 
 def run_command():
     ClusterCreate().run()
@@ -569,6 +603,7 @@ class ClusterCreate(ClusterCommand):
             self.storage_account = StorageAccount.storage_account_for_cluster(config["name"])
 
         self.resource_group = config["resource_group"]
+        self.deployment_name = "%s-azure-deployment" % config["name"]
 
         self.location = config["azure_location"]
         inbound_rules = self.generate_inbound_traffic_rules(config["inbound_rule"])
@@ -618,6 +653,9 @@ class ClusterCreate(ClusterCommand):
             self.existing_subnet_id = subnet.id
             config["subnet_cidr"] = cidr(subnet.address_prefix)
 
+            if not config.is_item_set_explicitly("head_node_ip"):
+                config["head_node_ip"] = config["subnet_cidr"][-2]
+
         if config["head_node_ip"] not in config["subnet_cidr"]:
             # We can't put this validation on the parameters because we have to check if it's an existing VPC first
             raise CODException(
@@ -678,6 +716,8 @@ class ClusterCreate(ClusterCommand):
             deployment_parameters["subnet_id"] = {"value": self.existing_subnet_id}
             deployment_parameters["create_new_vnet"] = {"value": "false"}
 
+        deployment_parameters["create_public_ip"] = {"value": str(config["create_public_ip"]).lower()}
+
         deployment_properties = DeploymentProperties(
             mode=DeploymentMode.incremental,
             template=template,
@@ -689,7 +729,7 @@ class ClusterCreate(ClusterCommand):
         try:
             deployment_async_operation = self.azure_api.resource_client.deployments.begin_create_or_update(
                 resource_group_name=self.resource_group,
-                deployment_name="%s-azure-deployment" % config["name"],
+                deployment_name=self.deployment_name,
                 parameters=Deployment(properties=deployment_properties)
             )
             deployment_async_operation.wait()
@@ -704,23 +744,41 @@ class ClusterCreate(ClusterCommand):
                         location=config["azure_location"]
                     )
                 )
+            if "DeploymentFailed" in e.message:
+                try:
+                    for operation in self.azure_api.resource_client.deployment_operations.list(
+                        resource_group_name=self.resource_group,
+                        deployment_name=self.deployment_name
+                    ):
+                        if operation.properties.status_code.upper() != "OK":
+                            resource_name = operation.properties.target_resource.resource_name
+                            error_code = operation.properties.status_message.error.code
+                            error_message = operation.properties.status_message.error.message
+                            log.error(f"Deployment error for resource {resource_name}: ({error_code}) {error_message}")
+                except Exception as inner_e:
+                    log.error(f"Error while diagnosing deployment failure: {inner_e}")
+
             self.on_error(self.azure_api.resource_client)
             raise CODException("Template deployment failed: %s" % e.message, caused_by=e)
         except Exception as e:
             self.on_error(self.azure_api.resource_client)
             raise CODException("Template deployment failed: %s" % str(e), caused_by=e)
 
-        public_ip = self.azure_api.network_client.public_ip_addresses.get(
-            self.resource_group, "head-node-public-ip"
-        ).ip_address
-
         instance_id = self.azure_api.compute_client.virtual_machines.get(
             self.resource_group, "%s" % self.head_node
         ).vm_id
 
-        log.info("Head node IP: %s" % public_ip)
+        if config["create_public_ip"]:
+            public_ip = self.azure_api.network_client.public_ip_addresses.get(
+                self.resource_group, "head-node-public-ip"
+            ).ip_address
+            log.info("Head node IP: %s" % public_ip)
 
-        wait_for_cluster_to_be_ready(config, public_ip, config["version"])
+            wait_for_cluster_to_be_ready(config, public_ip, config["version"])
+        else:
+            public_ip = "N/A"
+            log.info("Cluster was created without a public IP. COD cannot wait for cmdaemon to be ready.")
+            log.info(f"Head node private IP: {config['head_node_ip']}")
 
         utils.cod_log(log, "Deployment finished successfully.", 100)
 
